"""
Adapter of minimal file-like interface (only .read(size) and .close()
are to be implemented) to the wider file-like interface.

Currently, the following operations are implemented
.readline()
.read()
.peek(size)
.read(size)
.close()

NOTE: fh passed to the constructor no longer may be used due to
buffering inside FileAdapter.
"""

class FileAdapter(object):
    BUFFER_SIZE = 65536

    def __init__(self, fh):
        self.__fh = fh
        self.__buffer = ""

    def readline(self):
        lfpos = self.__buffer.find("\n")
        while lfpos == -1 and self.__fh:
            checked = len(self.__buffer)
            chunk = self.__fh.read(self.BUFFER_SIZE)
            if chunk == "":
                self.__fh = None
                ret = self.__buffer
                self.__buffer = ""
                return ret
            self.__buffer += chunk
            lfpos = self.__buffer.find("\n", checked)

        part = self.__buffer[:lfpos+1]
        self.__buffer = self.__buffer[lfpos+1:]
        return part

    def read(self, size = None):
        if size == None:
            while self.__fh:
                self.__read_in_buffer(self.BUFFER_SIZE)

            part = self.__buffer
            self.__buffer = ""
            return part

        if len(self.__buffer) < size:
            self.__read_in_buffer(size - len(self.__buffer))

        part = self.__buffer[:size]
        self.__buffer = self.__buffer[size:]
        return part

    def __read_in_buffer(self, size):
        if self.__fh:
            size = (size + self.BUFFER_SIZE - 1) / self.BUFFER_SIZE * self.BUFFER_SIZE
            chunk = self.__fh.read(size)
            if chunk == "":
                self.__fh = None
            self.__buffer += chunk

    def peek(self, size):
        if len(self.__buffer) < size:
            self.__read_in_buffer(size - len(self.__buffer))

        return self.__buffer[:size]

    def close(self):
        if self.__fh:
            self.__fh.close()

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
