import sys, os, logging, string

import streaming_extractor
import unpacker
import log
import PacketServer
import osutil
import stacktrace
import backup

_logger = logging.getLogger("backup_extractor")


class BackupMimeExtractorHandler(PacketServer.PacketHandler):
    def __init__(self, request, client_address, server, pc):
        self.pc = pc
        PacketServer.PacketHandler.__init__(self, request, client_address, server)

    def write_encoded_response(self, l):
        self.write_response("\0".join(l))

    def write_error(self, error):
        self.write_encoded_response(["ERR", error])

    def write_success(self, l):
        self.write_encoded_response(["OK"] + l)

    def decode_packet(self, packet):
        res = string.split(packet, "\0")
        return res[0], res[1:]

    def processPacket(self, packet):
        (command, args) = self.decode_packet(packet)

        _logger.debug(command + "(" + (", ".join(args)) + ")")

        commandsArgsNum = { "untar": 2,
                            "db": 7,
                            "unpack": 2 }

        commands = { "untar": unpacker.TarUnpacker,
                     "db": unpacker.DbUnpacker,
                     "unpack": unpacker.PartUnpacker }

        if command == "stop":
            self.write_success([])
            raise PacketServer.StopServer(None)

        if command not in commands:
            self.write_error("Unknown command")
            return

        if len(args) != commandsArgsNum[command]:
            self.write_error("'%s' command requires %d arguments"
                             % (command, commandsArgsNum[command]))
            return

        (cid, args) = args[0], args[1:]

        def partProcessor(bundle):
            def realPartProcessor(fh):
                return commands[command](*args)(fh, cid)
            return bundle.process(realPartProcessor)

        try:
            ret = self.pc.processPart(cid, partProcessor)
        except Exception, e:
            _logger.exception(e)
            self.write_error("Runtime error during '%s': %s" % (command, str(e)))
            return

        _logger.info("'%s' finished. Response: %s" % (command, str(ret)))

        self.write_success(ret)

def run(session_dir, backup_file, socket, cids=None):
    _logger.info("backup-mime-extractor started")

    try:
        if os.path.exists(socket):
            try:
                os.unlink(socket)
            except OSError, e:
                sys.stderr.write("Unable to remove stale socket file: " + e.strerror)
                return False

        b = backup.Backup('file', backup_file, session_dir)
        pc = streaming_extractor.PartsCacher(b, cids)

        def backupMimeExtractorHandlerCreator(request, client_address, server):
            BackupMimeExtractorHandler(request, client_address, server, pc)

        server = PacketServer.PacketServer(socket, backupMimeExtractorHandlerCreator, 0, 0755)

    except Exception, e:
        _logger.critical("Runtime error in backup-mime-extractor")
        sys.stderr.write("Unable to start backup-mime-extractor: " + unicode(e.__class__)  + " " \
                         + unicode(e) + "\n" + stacktrace.stacktrace())
        return False

    try:
        try:
            server.serve_forever()
        finally:
            osutil.unlink_nothrow(socket)
    except PacketServer.StopServer:
        _logger.info("backup-extractor finished")
    except Exception, e:
        _logger.critical("Runtime error in backup-extractor: " + unicode(e.__class__) + " " \
                         + unicode(e) + "\n" + stacktrace.stacktrace())
