import select

#
# select.poll implementation based on select.select: select.poll is
# not available on MacOSX
#

def safedel(seq, val):
    try:
        seq.remove(val)
    except ValueError:
        pass

def tofd(fd):
    if isinstance(fd, int):
        return fd
    return fd.fileno()

class Poll(object):
    def __init__(self):
        self.ifds = []
        self.ofds = []
        self.efds = []

    def register(self, fd, eventmask):
        fd = tofd(fd)
        if eventmask & (select.POLLIN | select.POLLPRI):
            if self.ifds.count(fd) == 0:
                self.ifds.append(fd)
        if eventmask & select.POLLOUT:
            if self.ofds.count(fd) == 0:
                self.ofds.append(fd)

        if self.efds.count(fd) == 0:
            self.efds.append(fd)

    def unregister(self, fd):
        fd = tofd(fd)
        safedel(self.ifds, fd)
        safedel(self.ofds, fd)
        safedel(self.efds, fd)

    def poll(self, timeout = None):
        if timeout:
            (ifds, ofds, efds) = select.select(self.ifds, self.ofds, self.efds, int(timeout*1000))
        else:
            (ifds, ofds, efds) = select.select(self.ifds, self.ofds, self.efds)

        events = {}
        for ifd in ifds:
            events.setdefault(ifd, 0)
            events[ifd] |= select.POLLIN
        for ofd in ofds:
            events.setdefault(ofd, 0)
            events[ofd] |= select.POLLOUT
        for efd in efds:
            events.setdefault(efd, 0)
            events[efd] |= select.POLLERR

        return events.items()

def poll():
    return Poll()

try:
    select.poll
except AttributeError:
    select.poll = poll
    select.POLLIN = 1
    select.POLLPRI = 2
    select.POLLOUT = 4
    select.POLLERR = 8
    select.POLLHUP = 16

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
